import net.sf.flashlib.lang.NullPointerException;

import net.sf.flashlib.util.Observer;



/**
 * A Java-style <code>Observable</code> class used to represent the "subject"
 * of the Observer design pattern. Observers must implement the <code>Observer</code>
 * interface and register to observe the subject via <code>addObserver()</code>.
 * 
 * @author  Marc Reichelt
 * @version 0.0.1
 */
class net.sf.flashlib.util.Observable {
	
	/**
	 * Flag that indicates whether the subject has changed.
	 */
	private var changed : Boolean = false;
	
	/**
	 * The observers.
	 */
	private var observers : Array = null;
	
	
	
	/**
	 * Creates a new Observable with zero observers.
	 */
	public function Observable() {
		observers = new Array();
	}
	
	/**
	 * Adds an observer if it is not already observing this object.
	 * 
	 * @param  observer  The observer that should be added.
	 */
	public function addObserver(observer : Observer) : Void {
		if (observer == null) {
			throw new NullPointerException("observer must not be null.");
		}
		
		// don't add the observer if it already observes this object
		for (var i : Number = 0; i < observers.length; i++) {
			if (Observer(observers[i]) == observer) {
				return;
			}
		}
		
		// add observer
		observers.push(observer);
	}
	
	/**
	 * Delete an observer.
	 * 
	 * @param  observer  The observer that should be deleted.
	 */
	public function deleteObserver(observer : Observer) : Void {
		if (observer == null) {
			return;
		}
		
		for (var i : Number = 0; i < observers.length; i++) {
			if (Observer(observers[i]) == observer) {
				observers.slice(i, 1);
				return;
			}
		}
	}
	
	/**
	 * Delete all observers.
	 */
	public function deleteObservers() : Void {
		observers = new Array();
	}
	
	/**
	 * Returns the number of observers.
	 * 
	 * @return  Number of observers.
	 */
	public function countObservers() : Number {
		return observers.length;
	}
	
	/**
	 * Notify all observers that the subject has changed.
	 * 
	 * @param  obj  An object containing arbitrary data to pass to observers.
	 */
	public function notifyObservers(obj : Object) : Void {
		// no changes - we don't need to notify our observers.
		if (!changed) {
			return;
		}
		
		// make a copy of observers object (ensures that the array does not
		// change while we are processing it)
		var observersCopy : Array = observers.slice(0);
		
		clearChanged();
		
		// notify observers
		for (var i : Number = 0; i < observersCopy.length; i++) {
			Observer(observersCopy[i]).update(this, obj);
		}
	}
	
	/**
	 * Check if the subject has changed.
	 * 
	 * @return  true if the subject has changed, otherwise false.
	 */
	public function hasChanged() : Boolean {
		return changed;
	}
	
	/**
	 * Indicates that the subject has either not changed or
	 * has notified its observers of the last change.
	 */
	private function clearChanged() : Void {
		changed = false;
	}
	
	/**
	 * Indicates that the subject has changed.
	 */
	private function setChanged() : Void {
		changed = true;
	}
	
}
