import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Hashtable;



/**
 * Creates a SWF file for a skin or all skins.
 * 
 * @author  Marc Reichelt
 * @version 0.0.4
 */
public class CreateSkin {
	
	/**
	 * Directory for basic sources.
	 */
	private static final File SRC_DIRECTORY = new File(".." + File.separator + "src");
	
	/**
	 * Directory for skins.
	 */
	private static final File SKINS_DIRECTORY = new File(".." + File.separator + "skins");
	
	/**
	 * The command for the MTASC binary.
	 */
	private static final String MTASC_COMMAND = getBinary("mtasc");
	
	/**
	 * The command for the SWFMILL binary.
	 */
	private static final String SWFMILL_COMMAND = getBinary("swfmill");
	
	
	
	/**
	 * Compile all ActionScript classes and create SWF file (including
	 * movie clips and images).
	 */
	public static void main(String[] args) {
		
		Hashtable<String, File> skins = getAllSkins();
		ArrayList<String> skinNames = new ArrayList<String>(skins.keySet());
		Collections.sort(skinNames);
		
		if (args.length == 0) {
			
			// display help
			
			System.out.println("Usage: java CreateSkin <skin>");
			System.out.println("        (to create the appropriate skin) or");
			System.out.println("       java CreateSkin ALL");
			System.out.println("        (to create all skins)");
			System.out.println("Available skins are:");
			
			for (String skinName : skinNames) {
				System.out.println(" " + skinName);
			}
			
			System.exit(0);
		}
		
		if (args[0].equals("ALL")) {
			// compile all skins
			
			for (String skinName : skinNames) {
				System.out.println("Working on: " + skinName);
				compileSkin(skins.get(skinName));
			}
		} else {
			// compile one skin
			
			if (skins.containsKey(args[0])) {
				compileSkin(skins.get(args[0]));
			} else {
				System.out.println("Skin not found: " + args[0]);
			}
		}
		
	}
	
	/**
	 * Execute a specific command.
	 * 
	 * @param  cmdarray          The command (with arguments) that will be executed.
	 * @param  workingDirectory  The working directory in that the command will be executed.
	 */
	private static void exec(String[] cmdarray, File workingDirectory) {
		try {
			
			Process child = Runtime.getRuntime().exec(cmdarray, null, workingDirectory);
			
			int c;
			InputStream in;
			
			// Get the input stream and read from it
			in = child.getInputStream();
			while ((c = in.read()) != -1) {
				System.out.print((char) c);
			}
			in.close();
			
			// Get the error stream and read from it
			in = child.getErrorStream();
			while ((c = in.read()) != -1) {
				System.out.print((char) c);
			}
			in.close();
			
			// wait for the process to exit
			child.waitFor();
			int exitValue = child.exitValue();
			
			if (exitValue != 0) {
				System.err.println("Process returned error code: " + exitValue);
				System.exit(1);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	/**
	 * Compiles a skin.
	 * 
	 * @param skinDir The skin directory.
	 */
	private static void compileSkin(File skinDir) {
		String skinName = skinDir.getName();
		
		try {
			File inputSWF = new File(skinDir.getCanonicalPath() + File.separator + "skin.swf");
			File inputXML = new File(skinDir.getCanonicalPath() + File.separator + "skin.xml");
			File outputSWF = new File(SKINS_DIRECTORY.getCanonicalPath() + File.separator + "emff_" + skinName + ".swf");
			ArrayList<String> command;
			
			// remove output SWF file if it exists
			if (outputSWF.isFile()) {
				if (!outputSWF.delete()) {
					throw new RuntimeException(outputSWF.getCanonicalPath() + " could not be deleted.");
				}
			}
			
			if (inputSWF.isFile() && inputSWF.canRead()) {
				// a skin swf file does exist already, so we don't have to create one
				// we just need to make a copy of it
				try {
					byte[] buffer = new byte[ 0xFFFF ];
					FileInputStream fis = new FileInputStream(inputSWF);
					FileOutputStream fos = new FileOutputStream(outputSWF);
	      			for (int length; (length = fis.read(buffer)) != -1; ) {
	        			fos.write(buffer, 0, length);
	        		}
	        	} catch (Exception e) {
	        		throw new RuntimeException(inputSWF.getCanonicalPath() + " could not be copied to " + outputSWF.getCanonicalPath() + ".");
	        	}
			} else {
				if (inputXML.isFile() && inputXML.canRead()) {
					// At first we create a SWF file that simply does nothing (no ActionScript code)
					// but contains all ressources we want to access with ActionScript later.
					command = new ArrayList<String>();
					
					command.add(SWFMILL_COMMAND);
					command.add("simple");
					command.add("skin.xml");
					command.add(outputSWF.getCanonicalPath());
					
					exec((String[]) command.toArray(new String[0]), skinDir);
				} else {
					throw new RuntimeException("No skin.xml for skin " + skinName + ".");
				}
			}
			
			
			
			// Now that the SWF file is created we add our ActionScript code to it.
			command = new ArrayList<String>();
			
			command.add(MTASC_COMMAND);
			command.add("-cp");
			command.add(SRC_DIRECTORY.getCanonicalPath());
			command.add("-version");
			command.add("7");
			command.add("-strict");
			command.add("-wimp");
			command.add("-main");
			command.add("Skin.as");
			command.add("-swf");
			command.add(outputSWF.getCanonicalPath());
			
			exec((String[]) command.toArray(new String[0]), skinDir);
		} catch (Exception e) {
			System.err.println("Error: " + e.getMessage());
			System.exit(1);
		}
	}
	
	/**
	 * Gets the string representation for a specific program.
	 * 
	 * @param name The program name.
	 * @return String representation for the program (e.g. "mtasc" under Linux or
	 *          "mtasc.exe" under Windows). 
	 */
	private static String getBinary(String name) {
		String binary = null;
		
		// detect platform
		if (System.getProperty("os.name").contains("Windows")) {
			binary = name + ".exe";
		} else {
			binary = name;
		}
		
		return binary;
	}
	
	/**
	 * Looks for all available skins.
	 * 
	 * @return All skin directories in the directory "skins".
	 */
	public static Hashtable<String, File> getAllSkins() {
		Hashtable<String, File> skins = new Hashtable<String, File>();
		
		if (SKINS_DIRECTORY.isDirectory()) {
			for (File file : SKINS_DIRECTORY.listFiles()) {
				try {
					if (file.isDirectory() && file.getName().matches("^[a-z0-9\\_]+$")) {
						File infoFile = new File(file.getCanonicalPath() + File.separator + "SKIN");
						if (infoFile.isFile() && infoFile.canRead()) {
							skins.put(file.getName(), file);
						}
					}
				} catch (Exception e) {
					
				}
			}
		}
		
		return skins;
	}
	
}
